<%# frozen_string_literal: true %>
<html>
    <head>
        <title>This is the parent frame title</title>
    </head>
    <body id="parentBody">
      <iframe src='/frame_child' id='childFrame'></iframe>
    </body>
</html>
