<%# frozen_string_literal: true %>
<form action="/form" method="post">
  <table id="agent_table">
    <caption>Agent</caption>

    <tr>
      <td>
        <label for="form_agent_name">Name</label>
      </td>
      <td>
        <input type="text" name="form[agent_name]" value="James" id="form_agent_name"/>
      </td>
    </tr>

    <tr>
      <td colspan="2">
        <input type="submit" value="Create"/>
      </td>
    </tr>
  </table>
</form>

<form action="/form" method="post">
  <table id="girl_table">
    <caption>Girl</caption>

    <tr>
      <td>
        <label for="form_girl_name">Name</label>
      </td>
      <td>
        <input type="text" name="form[girl_name]" value="Vesper" id="form_girl_name"/>
      </td>
    </tr>

    <tr>
      <td colspan="2">
        <input type="submit" value="Create"/>
      </td>
    </tr>
  </table>
</form>

<form action="/form" method="post">
  <table id="villain_table">
    <caption>Villain</caption>

    <tr>
      <td>
        <label for="form_villain_name">Name</label>
      </td>
      <td>
        <input type="text" name="form[villain_name]" value="Ernst" id="form_villain_name"/>
      </td>
    </tr>

    <tr>
      <td colspan="2">
        <input type="submit" value="Create"/>
      </td>
    </tr>
  </table>
</form>
