<%# frozen_string_literal: true %>
<h1>Postback</h1>

<form method="get">
  <p>
    <input type="submit" value="With no action">
  </p>
</form>

<form action="" method="get">
  <p>
    <input type="submit" value="With blank action">
  </p>
</form>
