<%# frozen_string_literal: true %>
<html>
    <head>
        <title>With Frames</title>
    </head>
    <body>
        <div id="divInMainWindow">This is the text for divInMainWindow</div>
    <iframe src="/frame_one" id="frameOne"></iframe>
    <iframe src="/frame_two" id="frameTwo"></iframe>
    <iframe src="/frame_parent" id="parentFrame"></iframe>
    </body>
</html>
