<div id="selectors">
  <h2>Built-in Selectors</h2>
  <ul>
    <% @selectors.each do |name, selector| %>
      <li>
        <h3>:<%= name %></h3>
        <div class="docstring">
          <div class="discussion">
            <p><%= h(selector.docstring) %></p>
          </div>
        </div>
        <div class="tags">
          <p class="inline">Locator:</p>
          <% if locator=selector.tag('locator') %>
            <p class="inline"><%= h locator.text %></p>
          <% end %>
          <% if selector.has_tag?('filter') %>
            <p>Filters:</p>
            <ul class="param">
              <% selector.tags('filter').each do |filter| %>
              <li>
                <span class="name"><%= filter.name %></span>
                <% if filter.types  %>
                <span class="type">(<tt><%= h(filter.types.join(', ')) %></tt>)</span>
                <% end %>
                <% if filter.text %>
                —
                <div class="inline"><p><%= h filter.text %></p></div>
                <% end %>
              </li>
              <% end %>
            </ul>
          <% end %>
        </div
      </li>
    <% end %>
  </ul>
</div>
