<%# frozen_string_literal: true %>
<html>
    <head>
        <title>This is the title of frame two</title>
    </head>
    <body>
        <div id="divInFrameTwo">This is the text of divInFrameTwo</div>
    </body>
</html>
