<%# frozen_string_literal: true %>
<html>
    <head>
        <title>This is the child frame title</title>
        <script>
        function closeWin() {
          var iframe = window.parent.document.getElementById('childFrame')
          iframe.parentNode.removeChild(iframe)
        }
        </script>
    </head>
    <body id="childBody">
      <a href="javascript:void(0)" onClick="closeWin()">Close Window</a>
      <iframe src="/frame_one" id="grandchildFrame1"></iframe>
      <iframe src="/frame_two" id="grandchildFrame2"></iframe>
    </body>
</html>
