<%# frozen_string_literal: true %>
<html xmlns="http://www.w3.org/1999/xhtml" lang="en">
  <body>
    <div>
      <a href="#">First Link</a>
    </div>
    Text node
    <div>
      <a href="#">Second Link</a>
      <a href="#">Third Link</a>
    </div>
  </body>
</html>
