<%# frozen_string_literal: true %>
<html>
    <head>
        <title>Title of the first popup</title>
    </head>
    <body>
        <div id="divInPopupOne">This is the text of divInPopupOne</div>
    </body>
</html>
