<%# frozen_string_literal: true %>
<html xmlns="http://www.w3.org/1999/xhtml" lang="en">
  <head>
    <meta http-equiv="Content-type" content="text/html; charset=utf-8"/>
    <base href="http://example2.com" />
    <title>with_external_source</title>
  </head>
  <body id="with_js">
    <h1>FooBar</h1>
  </body>
</html
