<%# frozen_string_literal: true %>
<form action="/form" method="post">
  <fieldset id="agent_fieldset">
    <legend>Agent</legend>
  
    <p>
      <label for="form_agent_name">Name</label>
      <input type="text" name="form[agent_name]" value="James" id="form_agent_name"/>
    </p>
  
    <p>
      <input type="submit" value="Create"/>
    </p>
  </fieldset>
</form>

<form action="/form" method="post">
  <fieldset id="villain_fieldset">
    <legend>Villain</legend>
  
    <p>
      <label for="form_villain_name">Name</label>
      <input type="text" name="form[villain_name]" value="Ernst" id="form_villain_name"/>
    </p>
  
    <p>
      <input type="submit" value="Create"/>
    </p>
  </fieldset>
</form>
