<%# frozen_string_literal: true %>
<p>
  <a href="/host">Relative Host</a>
  <a href="<%= params[:absolute_host] %>/host">Absolute Host</a>
</p>

<form action="/host">
  <p><input type="submit" value="Relative Host"/></p>
</form>

<form action="<%= params[:absolute_host] %>/host">
  <p><input type="submit" value="Absolute Host"/></p>
</form>
