<%# frozen_string_literal: true %>
<html>
    <head>
        <title>This is the title of frame one</title>
    </head>
    <body>
        <div id="divInFrameOne">This is the text of divInFrameOne</div>
    </body>
</html>
